#!/usr/bin/env python3

DOCTYPE = """
<!DOCTYPE html>"""
XMLNS = """
<html lang="en">"""
HEAD = """
<head>
  <meta charset="UTF-8">
  <meta http-equiv='Content-Type' content='text/html; charset=utf-8'>
  <meta name="viewport" content="width=device-width, initial-scale=1.0, maximum-scale=1.0, user-scalable=0" />
  <meta name='title' content='OSS Notice'>
    <title>OSS Notice</title>"""
STYLE_CSS = """
        <style>
        body, th, td, div, textarea, pre {
          font-family: 'Malgun Gothic','맑은 고딕','Apple SD Gothic Neo','돋움',dotum,sans-serif,Arial;
        }
        a,a:visited,a:active { color: #0969da ;text-decoration: none}
        a:hover { color:dodgerblue ; text-decoration: underline}
        .notice { position: relative;}
        h2 {
          color:#ffffff;
          font-size:26px;
          text-align:center;
          position: relative;
          -webkit-box-sizing: border-box;
          box-sizing: border-box;
        }
        h3 {
          /*color:#ea002c;*/
          font-size:22px;
          color:#000;
        }
        h4 {
          font-size:18px;
          font-weight: 600;
        }
        h4 a {
          font-size:14px;
          display:inline-block;
          /*background:#fae100;*/
          border: 1px solid #8a8a8a;
          padding:4px 12px;
          border-radius: 16px;
          margin:4px;
        }
        h4 a,a:visited { color: #0969da ;text-decoration: none}
        h4 a:hover , h4 a:active { color:#ffffff ; text-decoration: none}
        h4 a:hover{
          background:#0969da;
          cursor:pointer;
        }
        .article {
          color:#777777;
          font-weight: 400;
          font-size:14px;
          line-height:160%;
          margin-bottom:44px;
        }
        .tt_wrap {
          padding:24px;
          background-color:#000000;
        }
        .tt_desc {
          color:#202021;
          font-size:16px;
          padding:24px 36px;
          line-height:150%;
        }
        .tt_line {border-top: #dcdcdc solid 1px;font-size: 1px;}
        .tt_licenses, .tt_sc {
          padding: 24px;
        }
        .tt_licenses .license {
          border-top: 1px solid #000;
        }
        .no_data {
          text-align:center;
          padding: 30px !important;
        }
        .tt_sc {
          background-color:#eeeeee;
        }
        .tt_sc .sc_title {
          color:#202021;
          font-size:22px;
          font-weight: 600;
        }
        .sc_desc , .sc_contact {
          font-size:14px;
          line-height:150%;
          margin-top:25px;
        }
        .tt_components {
          position: relative;
          overflow: hidden;
          margin: 0 auto;
          padding: 24px;
        }
        .tt_components table {
          display: table;
          padding: 24px;
          width: 100%;
          border-collapse: collapse;
          border-spacing: 0;
          border-top: 2px solid #000;
          border-bottom: 1px solid #000;
        }
        .tt_components thead th {
          font-size:16px;
          text-align:left;
          border-left: 1px solid #eaecf0;
          border-bottom:1px solid #eaecf0;
        }
        .tt_components td {
          border-bottom:#eaecf0 solid 1px;
          border-left: 1px solid #eaecf0;
          font-size:14px;
        }
        .tt_components tr td:first-child , .tt_components thead th:first-child {
          border-left:none;
        }
        .tt_components tr:last-child td {
          border-bottom:none;
        }
        .tt_components table tbody tr td {
          border-bottom: 1px solid #efefef;
        }
        .tt_components table thead tr th,
        .tt_components table tbody tr td {
          padding: 12px 20px;
          font-size: 16px;
          line-height: 22px;
        }
        @media screen and (max-width: 990px) {
          .tt_components table col {
            width: 100% !important;
          }
          .tt_components table thead {
            display: none;
          }
          .tt_components table tbody tr {
            border-bottom: 1px solid #efefef;
          }
          .tt_components td {
            border-left:none;
          }
          .tt_components table tbody tr td {
            width: 100%;
            display: flex;
            margin-bottom: 2px;
            padding: 5px;
            border-bottom: none;
            font-size: 14px;
            line-height: 18px;
          }
          .tt_components table tbody tr td:first-child,
          .tt_components table tbody tr th:first-child {
            padding-top: 16px;
          }
          .tt_components table tbody tr td:last-child,
          .tt_components table tbody tr th:last-child {
            padding-bottom: 15px;
          }
          .tt_components table tbody tr td:before {
            margin-right: 12px;
            -webkit-box-flex: 0;
            -ms-flex: 0 0 100px;
            flex: 0 0 100px;
            font-size:16px;
            font-weight: 800;
            content: attr(data-label);
          }
        }
      </style>
    </head>"""
BODY_TABLE_1 = """
    <body>
    <div class="notice">
      <div class="tt_wrap">"""
BODY_TABLE_2 = """
      </div>
      <div class="tt_desc">"""
COMPONENT_TITLE = """
       </div>
      <div class="tt_line"></div>
       <div class="tt_components">
        <h3>components</h3>
        <table>
          <colgroup>
            <col style="width:20%">
            <col style="width:30%">
            <col style="width:50%">
          </colgroup>
        <thead>
        <tr>
          <th>Name</th>
          <th>License</th>
          <th>Copyright</th>
        </tr>
        </thead>
        <tbody>"""
TABLE_ROW_NO = """
        <tr>
          <td colspan="3" class="no_data">
            No Components
          </td>
        </tr>"""
TABLE_CLOSE = """
        </tbody>
        </table>
    </div>
"""
LICENSE_TITLE = """
    <div class="tt_licenses">
        <h3>licenses</h3>
"""
LICENSE_TABLE_CLOSE = """</div>"""
WRITTEN_OFFER_TITLE = """
    <div class="tt_line"></div>
    <div class="tt_sc">
        <div class="sc_title">Offer of Source Code</div>
        <div class="sc_desc">
"""
WRITTEN_OFFER_CLOSE = """
        </div>
        <div class="sc_contact">
"""
FOOTER = """
        </div>
    </div>
</div>
</body>
"""



#!/usr/bin/env python3

# SPDX-FileCopyrightText: Copyright 2022 SK TELECOM CO., LTD. <haksung@sk.com>
# SPDX-FileCopyrightText: Copyright (c) 2022 Kakao Corp. https://www.kakaocorp.com
#
# SPDX-License-Identifier: Apache-2.0

import openpyxl
import pandas as pd
from onot.parsing import parser

# sheets
SHEET_DOCUMENT_INFO = "Document Info"
SHEET_PACKAGE_INFO = "Package Info"
SHEET_EXTRACTED_LICENSE_INFO = "Extracted License Info"
SHEET_PER_FILE_INFO = "Per File Info"

# columns
COLUMN_DOCUMENT_NAME = "Document Name"
COLUMN_CREATOR = "Creator"
COLUMN_PACKAGE_NAME = "Package Name"
COLUMN_PACKAGE_VERSION = "Package Version"
COLUMN_PACKAGE_DOWNLOAD_LOCATION = "Package Download Location"
COLUMN_LICENSE_CONCLUDED = "License Concluded"
COLUMN_LICENSE_DECLARED = "License Declared"
COLUMN_PACKAGE_COPYRIGHT_TEXT = "Package Copyright Text"
COLUMN_IDENTIFIER = "Identifier"
COLUMN_EXTRACTED_TEXT = "Extracted Text"
COLUMN_LICENSE_NAME = "License Name"
COLUMN_FILE_NAME = "File Name"
COLUMN_LICENSE_INFO_IN_FILE = "License Info in File"
COLUMN_FILE_COPYRIGHT_TEXT = "File Copyright Text"
COLUMN_ARTIFACT_OF_HOMEPAGE = "Artifact of Homepage"


class Parser(parser.AbstractParser):

    def __init__(self, file):
        print("debug:" + "excel Parser class")
        super().__init__()
        self.wb = openpyxl.load_workbook(file)

    def validate_file(self, file):
        # validate file contents : all necessary information should be in the file.
        # sheet & columns
        # 1. Document Info sheet
        # - Document Name
        # - Creator (Organization / Email)
        # 2. Package Info
        # - Package Name
        # - Package Version
        # - Package Download Location
        # - License Concluded
        # - Package Copyright Text
        # 3. Per File Info
        # - File Name
        # - Artifact of Homepage
        # - License Concluded
        # - License Info in File
        # - File Copyright Text
        ws_names = self.wb.sheetnames
        print("debug: " + str(ws_names))
        required_sheets = [
            SHEET_DOCUMENT_INFO,
            SHEET_PACKAGE_INFO,
            SHEET_PER_FILE_INFO
            # SHEET_EXTRACTED_LICENSE_INFO,
        ]
        for sheet in required_sheets:
            if sheet not in ws_names:
                raise ValueError("required sheet is not existed: " + sheet)  

    def document_info(self):
        sheet = self.wb.get_sheet_by_name(SHEET_DOCUMENT_INFO)
        df = pd.DataFrame(sheet.values)
        df.columns = df.iloc[0, :]
        df = df.iloc[1:, :]
        
        # get document name
        document_name = df.loc[:, COLUMN_DOCUMENT_NAME].values.tolist()
        if document_name[0] is not None:
            self.doc["name"] = document_name[0]
        else:
            raise ValueError("required value is not existed: " + COLUMN_DOCUMENT_NAME) 
        
        # get organization & email 
        creator_info = df.loc[:, COLUMN_CREATOR].values.tolist()
        for creator in creator_info:
            if 'Organization' in creator:
                if '@' in creator:
                    organization_email = creator.replace('Organization: ', '').split('(')
                    organization = organization_email[0].strip()
                    email = organization_email[1].replace(')', '')
                    creation_info = {
                        "organization": organization,
                        "email": email, 
                    }
                    self.doc["creationInfo"] = creation_info
                else:
                    raise ValueError("email info is not existed.") 
        if "creationInfo" not in self.doc:
            raise ValueError("Organization info is not existed in the " + SHEET_DOCUMENT_INFO) 

    def package_info(self):
        sheet = self.wb.get_sheet_by_name(SHEET_PACKAGE_INFO)
        df = pd.DataFrame(sheet.values)
        df.columns = df.iloc[0, :]
        df = df.iloc[1:, :]
        
        # get package info
        # validate necessary columns are existed
        if COLUMN_PACKAGE_NAME not in df.columns:
            raise ValueError("required column is not existed: " + COLUMN_PACKAGE_NAME) 
        if COLUMN_PACKAGE_VERSION not in df.columns:
            raise ValueError("required column is not existed: " + COLUMN_PACKAGE_VERSION) 
        if COLUMN_LICENSE_CONCLUDED not in df.columns:
            raise ValueError("required column is not existed: " + COLUMN_LICENSE_CONCLUDED) 
        if COLUMN_LICENSE_DECLARED not in df.columns:
            raise ValueError("required column is not existed: " + COLUMN_LICENSE_DECLARED) 
        if COLUMN_PACKAGE_COPYRIGHT_TEXT not in df.columns:
            raise ValueError("required column is not existed: " + COLUMN_PACKAGE_COPYRIGHT_TEXT) 
        if COLUMN_PACKAGE_DOWNLOAD_LOCATION not in df.columns:
            raise ValueError("required column is not existed: " + COLUMN_PACKAGE_DOWNLOAD_LOCATION) 

        package_info_list = df.loc[:, 
            [
                COLUMN_PACKAGE_NAME, 
                COLUMN_PACKAGE_VERSION,
                COLUMN_LICENSE_CONCLUDED,
                COLUMN_LICENSE_DECLARED,
                COLUMN_PACKAGE_COPYRIGHT_TEXT,
                COLUMN_PACKAGE_DOWNLOAD_LOCATION,
                ]].values.tolist()
        self.packages = []
        for package_info in package_info_list:
            # None check
            for index, value in enumerate(package_info):
                if value is None:
                    package_info[index] = ''
            package = {
                "name": package_info[0],
                "versionInfo": str(package_info[1]),
                "licenseConcluded": self.remove_enclosed_parentheses(package_info[2]),
                "licenseDeclared": self.remove_enclosed_parentheses(package_info[3]),
                "copyrightText": package_info[4],
                "downloadLocation": package_info[5].replace("\"", ""),
            }
            self.append_package(package)

    def per_file_info(self):
        sheet = self.wb.get_sheet_by_name(SHEET_PER_FILE_INFO)
        df = pd.DataFrame(sheet.values)
        df.columns = df.iloc[0, :]
        df = df.iloc[1:, :]

        if COLUMN_FILE_NAME not in df.columns:
            raise ValueError("required column is not existed: " + COLUMN_FILE_NAME)
        if COLUMN_LICENSE_CONCLUDED not in df.columns:
            raise ValueError("required column is not existed: " + COLUMN_LICENSE_CONCLUDED)
        if COLUMN_LICENSE_INFO_IN_FILE not in df.columns:
            raise ValueError("required column is not existed: " + COLUMN_LICENSE_INFO_IN_FILE)
        if COLUMN_FILE_COPYRIGHT_TEXT not in df.columns:
            raise ValueError("required column is not existed: " + COLUMN_FILE_COPYRIGHT_TEXT)
        if COLUMN_ARTIFACT_OF_HOMEPAGE not in df.columns:
            raise ValueError("required column is not existed: " + COLUMN_ARTIFACT_OF_HOMEPAGE)

        per_file_info_list = df.loc[:,
            [
                COLUMN_FILE_NAME,
                COLUMN_LICENSE_CONCLUDED,
                COLUMN_LICENSE_INFO_IN_FILE,
                COLUMN_FILE_COPYRIGHT_TEXT,
                COLUMN_ARTIFACT_OF_HOMEPAGE
            ]].values.tolist()
        for per_file_info in per_file_info_list:
            for index, value in enumerate(per_file_info):
                if value is None:
                    per_file_info[index] = ''

            package_name, package_version = self.extract_package_name_and_package_version(per_file_info[0])
            package = {
                "name": package_name,
                "versionInfo": str(package_version),
                "licenseConcluded": self.remove_enclosed_parentheses(per_file_info[1]),
                "licenseDeclared": self.remove_enclosed_parentheses(per_file_info[2]),
                "copyrightText": per_file_info[3],
                "downloadLocation": str(per_file_info[4]).replace("\"", "")
            }
            self.append_package(package)

    def extracted_license_info(self):
        sheet = self.wb.get_sheet_by_name(SHEET_EXTRACTED_LICENSE_INFO)
        df = pd.DataFrame(sheet.values)
        df.columns = df.iloc[0, :]
        df = df.iloc[1:, :]
        
        # get package info
        # validate necessary columns are existed
        if COLUMN_IDENTIFIER not in df.columns:
            raise ValueError("required column is not existed: " + COLUMN_IDENTIFIER) 
        if COLUMN_EXTRACTED_TEXT not in df.columns:
            raise ValueError("required column is not existed: " + COLUMN_EXTRACTED_TEXT) 
        if COLUMN_LICENSE_NAME not in df.columns:
            raise ValueError("required column is not existed: " + COLUMN_LICENSE_NAME) 
        
        extracted_license_list = df.loc[:, 
            [
                COLUMN_IDENTIFIER, 
                COLUMN_EXTRACTED_TEXT,
                COLUMN_LICENSE_NAME,
                ]].values.tolist()
        self.extracted_licenses = []
        for extracted_license_info in extracted_license_list:
            for index, value in enumerate(extracted_license_info):
                if value is None:
                    extracted_license_info[index] = ''

            extracted_license = {
                "identifier": extracted_license_info[0],
                "extractedText": extracted_license_info[1],
                "licenseName": extracted_license_info[2],
            }
            self.doc["extracted_license"].append(extracted_license)



#!/usr/bin/env python3
import pprint

# SPDX-FileCopyrightText: Copyright 2022 SK TELECOM CO., LTD. <haksung@sk.com>
# SPDX-FileCopyrightText: Copyright (c) 2022 Kakao Corp. https://www.kakaocorp.com
#
# SPDX-License-Identifier: Apache-2.0

import os
from rdflib import Graph, Namespace, RDF
from rdflib.term import Literal, URIRef, BNode
from onot.parsing import parser

SUBJECT_DOCUMENT = "SpdxDocument"
SUBJECT_PACKAGE = "Package"
SUBJECT_FILE = "File"
SUBJECT_EXTRACTED_LICENSE = "ExtractedLicensingInfo"

PREDICATE_DOCUMENT_NAME = "name"
PREDICATE_DOCUMENT_CREATOR = "creator"

PREDICATE_PACKAGE_NAME = "name"
PREDICATE_PACKAGE_VERSION_INFO = "versionInfo"
PREDICATE_PACKAGE_LICENSE_CONCLUDED = "licenseConcluded"
PREDICATE_PACKAGE_LICENSE_DECLARED = "licenseDeclared"
PREDICATE_PACKAGE_COPYRIGHT_TEXT = "copyrightText"
PREDICATE_PACKAGE_DOWNLOAD_LOCATION = "downloadLocation"

PREDICATE_FILE_NAME = "fileName"
PREDICATE_FILE_LICENSE_CONCLUDED = "licenseConcluded"
PREDICATE_FILE_LICENSE_DECLARED = "licenseInfoInFile"
PREDICATE_FILE_COPYRIGHT_TEXT = "copyrightText"
PREDICATE_FILE_DOWNLOAD_LOCATION = ""

PREDICATE_LICENSE_ID = "licenseId"
PREDICATE_LICENSE_NAME = "name"
PREDICATE_LICENSE_EXTRACTED_TEXT = "extractedText"

LICENSE_COMPOSITE_OPERATORS = {
    "DisjunctiveLicenseSet": " OR ",
    "ConjunctiveLicenseSet": " AND ",
    "WithExceptionOperator": " WITH "
}

class Parser(parser.AbstractParser):
    def __init__(self, file):
        print("debug:" + "RDF/XML Parser class")
        super().__init__()
        self.graph = Graph().parse(source=file, format="xml")
        self.spdx_namespace = Namespace("http://spdx.org/rdf/terms#")

    def validate_file(self, file):
        # validate file contents : all necessary information should be in the file.
        # subject & predicates
        # 1. SpdxDocument Subject
        # - name
        # - creator (Organization / Email)
        # 2. Package Subject
        # - name
        # - versionInfo
        # - downloadLocation
        # - licenseConcluded
        # - licenseDeclared
        # - copyrightText
        # 3. File Subject
        # - fileName
        # - licenseConcluded
        # - licenseInfoInFile
        # - copyrightText
        # - downloadLocation
        required_subjects = [
            SUBJECT_DOCUMENT,
            SUBJECT_PACKAGE,
            SUBJECT_PACKAGE,
            # SUBJECT_EXTRACTED_LICENSE
        ]
        for subject in required_subjects:
            if not (None, RDF.type, self.spdx_namespace[subject]) in self.graph:
                raise ValueError("required subject is not existed: " + subject)

    def validate_predicates_exist(self, subject, predicates):
        # validate necessary predicate are existed
        for predicate in predicates:
            if not (subject, self.spdx_namespace[predicate], None) in self.graph:
                raise ValueError("required predicate is not existed: " + self.spdx_namespace[predicate])

    def extract_license_expression(self, object):
        if type(object) == Literal:
            return str(object)
        elif type(object) == URIRef:
            # parse license name from url
            basename = os.path.basename(object)
            return basename.split("#")[1] if "#" in basename else basename
        # If the object is BNode(Blank Node), there are more than two objects.
        # https://rdflib.readthedocs.io/en/stable/rdf_terms.html#bnodes
        # This case means dual license.
        elif type(object) == BNode:
            for _, _, tag_name in self.graph.triples((object, RDF.type, None)):
                operator_tag = str(os.path.basename(tag_name)).split("#")[1]
                operator = LICENSE_COMPOSITE_OPERATORS[operator_tag]

            license_expression = ""
            for _, _, o in self.graph.triples((object, self.spdx_namespace["member"], None)):
                license_expression = license_expression + self.extract_license_expression(o) + operator

            for _, _, o in self.graph.triples((object, self.spdx_namespace["licenseException"], None)):
                license_expression = license_expression + self.extract_license_expression(o) + operator

            return license_expression[:-len(operator)] if operator == " WITH " else "(" + license_expression[:-len(operator)] + ")"

    def extract(self, subject, predicates):
        extracted = {}
        for predicate in predicates:
            objects = []
            for s, p, o in self.graph.triples((subject, self.spdx_namespace[predicate], None)):
                if predicate in [
                    PREDICATE_PACKAGE_LICENSE_CONCLUDED,
                    PREDICATE_PACKAGE_LICENSE_DECLARED,
                    PREDICATE_FILE_LICENSE_CONCLUDED,
                    PREDICATE_FILE_LICENSE_DECLARED
                ]:
                    license_expression = self.extract_license_expression(o)
                    objects.append(self.remove_enclosed_parentheses(license_expression))
                elif type(o) == BNode:
                    objects.append(o)
                else:
                    objects.append(str(o).replace("\n", "").strip())

            if len(objects) == 0:
                extracted[predicate] = ""
            elif len(objects) == 1:
                extracted[predicate] = objects[0]
            else:
                extracted[predicate] = objects
        return extracted

    def document_info(self):
        # get document name
        for subject, _, _ in self.graph.triples((None, RDF.type, self.spdx_namespace["SpdxDocument"])):
            self.validate_predicates_exist(subject, [PREDICATE_DOCUMENT_NAME])
            document_info = self.extract(subject, [PREDICATE_DOCUMENT_NAME])
            self.doc["name"] = document_info[PREDICATE_DOCUMENT_NAME]

        # get organization & email
        for subject, _, _ in self.graph.triples((None, RDF.type, self.spdx_namespace["CreationInfo"])):
            self.validate_predicates_exist(subject, [PREDICATE_DOCUMENT_CREATOR])
            creators = self.extract(subject, [PREDICATE_DOCUMENT_CREATOR])
            for creator in creators[PREDICATE_DOCUMENT_CREATOR]:
                if 'Organization' in creator:
                    if '@' in creator:
                        organization_email = creator.replace('Organization: ', '').split('(')
                        organization = organization_email[0].strip()
                        email = organization_email[1].replace(')', '')
                        creation_info = {
                            "organization": organization,
                            "email": email,
                        }
                        self.doc["creationInfo"] = creation_info
                    else:
                        raise ValueError("email info is not existed.")

    def package_info(self):
        required_predicates = [
            PREDICATE_PACKAGE_NAME,
            PREDICATE_PACKAGE_VERSION_INFO,
            PREDICATE_PACKAGE_LICENSE_CONCLUDED,
            PREDICATE_PACKAGE_LICENSE_DECLARED,
            PREDICATE_PACKAGE_COPYRIGHT_TEXT,
            PREDICATE_PACKAGE_DOWNLOAD_LOCATION
        ]
        non_required_predicates = []

        for subject, _, _ in self.graph.triples((None, RDF.type, self.spdx_namespace["Package"])):
            self.validate_predicates_exist(subject, required_predicates)
            package_info = self.extract(subject, required_predicates + non_required_predicates)
            package = {
                "name": package_info[PREDICATE_PACKAGE_NAME],
                "versionInfo": package_info[PREDICATE_PACKAGE_VERSION_INFO],
                "licenseConcluded": package_info[PREDICATE_PACKAGE_LICENSE_CONCLUDED],
                "licenseDeclared": package_info[PREDICATE_PACKAGE_LICENSE_DECLARED],
                "copyrightText": package_info[PREDICATE_PACKAGE_COPYRIGHT_TEXT],
                "downloadLocation": package_info[PREDICATE_PACKAGE_DOWNLOAD_LOCATION]
            }
            self.append_package(package)

    def per_file_info(self):
        required_predicates = [
            PREDICATE_FILE_NAME,
            PREDICATE_FILE_LICENSE_CONCLUDED,
            PREDICATE_FILE_LICENSE_DECLARED,
            PREDICATE_FILE_COPYRIGHT_TEXT,
        ]
        non_required_predicates = [
            PREDICATE_FILE_DOWNLOAD_LOCATION
        ]

        for subject, _, _ in self.graph.triples((None, RDF.type, self.spdx_namespace["File"])):
            self.validate_predicates_exist(subject, required_predicates)
            file_info = self.extract(subject, required_predicates + non_required_predicates)

            package_name, package_version = self.extract_package_name_and_package_version(file_info[PREDICATE_FILE_NAME])
            package = {
                "name": package_name,
                "versionInfo": package_version,
                "licenseConcluded": file_info[PREDICATE_FILE_LICENSE_CONCLUDED],
                "licenseDeclared": file_info[PREDICATE_FILE_LICENSE_DECLARED],
                "copyrightText": file_info[PREDICATE_FILE_COPYRIGHT_TEXT],
                "downloadLocation": ""
            }
            self.append_package(package)

    def extracted_license_info(self):
        required_predicates = [
            PREDICATE_LICENSE_ID,
            PREDICATE_LICENSE_EXTRACTED_TEXT
        ]
        non_required_predicates = [
            PREDICATE_LICENSE_NAME
        ]

        for subject, _, _ in self.graph.triples((None, RDF.type, self.spdx_namespace["ExtractedLicensingInfo"])):
            self.validate_predicates_exist(subject, required_predicates)
            license_info = self.extract(subject, required_predicates + non_required_predicates)
            extracted_license = {
                "identifier": license_info[PREDICATE_LICENSE_ID],
                "licenseName":  license_info[PREDICATE_LICENSE_NAME],
                "extractedText": license_info[PREDICATE_LICENSE_EXTRACTED_TEXT]
            }
            self.doc["extracted_license"].append(extracted_license)



#!/usr/bin/env python3

# SPDX-FileCopyrightText: Copyright 2022 SK TELECOM CO., LTD. <haksung@sk.com>
# SPDX-FileCopyrightText: Copyright (c) 2022 Kakao Corp. https://www.kakaocorp.com
#
# SPDX-License-Identifier: Apache-2.0

import os
import re
from onot.generating.html_resource import *
from datetime import datetime

class Generator():

    def __init__(self):
        print("debug:" + "Html class")

    def convert_license_expression(self, license_name):
        splited = re.split(r'OR|AND|WITH', str(license_name))
        licenses = list(map(lambda license: license.replace("(", "").replace(")", "").strip(), splited))
        license_component = license_name
        for license in licenses:
            license_component = str(license_component).replace(str(license), '<a href="#' + str(license) + '">' + str(license) + '</a>')
        return license_component

    def convert_download_location(self, download_location, name_version):
        if download_location == "":
            return name_version
        else:
            return """
                <a href="
                """ + download_location + """ 
                " target="_blank">
                """ + name_version + "</a>"

    def make_html_code(self, doc):
        title = doc['name']
        intro = 'A portion of this ' +  \
            doc['creationInfo']['organization'] + \
            ' product contains open source software, which is used and distributed in accordance with the specific license under which the open source software is distributed. A list of such open source software and the corresponding license terms is as follows:'
        head = DOCTYPE + XMLNS + HEAD + STYLE_CSS
        body_title = BODY_TABLE_1 + '<h2>OSS Notice for ' + title + '</h2>'
        body_intro = BODY_TABLE_2 + intro

        # component info field
        body_component = COMPONENT_TITLE
        for component in doc['packages']:
            name_version = component['name'] + ' ' + str(component['versionInfo'])
            body_component += '<tr>'
            body_component += '<td scope="row" data-label="Name" id="' + name_version.replace(' ', '_') + '">'
            body_component += self.convert_download_location(component["downloadLocation"], name_version)
            body_component += '</td>'
            body_component += '<td data-label="License">' + self.convert_license_expression(component['licenseConcluded']) + '</td>'
            body_component += '<td data-label="Copyright">' + str(component['copyrightText']) + '</td>'
            body_component += '</tr>'
        body_component_close = TABLE_CLOSE

        # license text field
        body_license = LICENSE_TITLE
        for license in doc['licenses']:
            body_license += '<div class="license">'
            body_license += '<h4 id="' + license['licenseId'] + '">' + license['name']
            # component list
            for package in license['packages']:
                name_version = ' '.join(package)
                body_license += '<a href = "#' + name_version.replace(' ', '_') + '">' + name_version + '</a>'
            body_license += '</h4>'
            body_license += '<div class="article">'
            # license text
            license_text = ''
            # if license['licenseTextHtml'] is None:
            #     license_text = license['licenseText'].replace('\n', '<br>')
            # else:
            #     license_text = license['licenseTextHtml']
            license_text = license['licenseText'].replace('\n', '<br>')
            body_license += '<p>' + license_text + '</p><br />'
            body_license += '</div>'
            body_license += '</div>'
        body_license += LICENSE_TABLE_CLOSE

        # writter
        body_offer = WRITTEN_OFFER_TITLE
        email = doc['creationInfo']['email']
        body_offer += 'This product includes software code developed by third parties, including software code subject to the GNU General Public License ("GPL") or GNU Lesser General Public License ("LGPL"). Where such specific license terms entitle you to the source code of such software, we will provide upon written request the applicable GPL and LGPL source code files via CD-ROM for a nominal cost to cover shipping and media charges as allowed under the GPL and LGPL. Please direct all inquiries to '
        body_offer += email + '. You may obtain the complete Corresponding Source code from us for a period of three years after our last shipment of this product. This offer is valid to anyone in receipt of this information.'
        body_offer += "</p>\n" + WRITTEN_OFFER_CLOSE

        # closing
        body_closing = 'If you have any questions regarding open source software contained in this product, please contact '
        body_closing += email + '.'

        # footer
        footer = FOOTER

        return head + body_title + body_intro + body_component + body_component_close + body_license + body_offer + body_closing + footer

    def generate_html_file(self, doc, html_code):
        now = datetime.now()
        date_time = now.strftime("%Y%m%d_%H%M%S")
        name = doc['name'].replace(' ', '_')
        filename = 'OSS_Notice_' + doc['name'].replace(' ', '_') + '_' + date_time + '.html'
        filepathname = os.path.join('output', filename)
        f = open(filepathname, 'w')
        f.write(html_code)
        f.close()
        print("debug: output is here - " + str(filepathname))

    def generate(self, doc):
        html_code = self.make_html_code(doc)
        self.generate_html_file(doc, html_code)
        print("debug: " + "generate completed")



#!/usr/bin/env python3

# SPDX-FileCopyrightText: Copyright 2022 SK TELECOM CO., LTD. <haksung@sk.com>
# SPDX-FileCopyrightText: Copyright (c) 2022 Kakao Corp. https://www.kakaocorp.com
#
# SPDX-License-Identifier: Apache-2.0

import requests

SPDX_LICENSE_URL_PREFIX = "https://spdx.org/licenses/"
SPDX_LICENSE_JSON_URL = "https://spdx.org/licenses/licenses.json"
SPDX_LICENSE_EXCEPTION_JSON_URL = "https://spdx.org/licenses/exceptions.json"

class SPDX_License():

    def __init__(self):
        print("debug:" + "SPDX_License")
        self.spdx_license_list = []
        self.spdx_license_exception_list = []

    def get_spdx_license_list(self):
        r = requests.get(SPDX_LICENSE_JSON_URL)
        self.spdx_license_list = r.json()

    def get_spdx_license_exception_list(self):
        r = requests.get(SPDX_LICENSE_EXCEPTION_JSON_URL)
        self.spdx_license_exception_list = r.json()
    
    def get_spdx_license_detailsUrl(self, license_id):
        print("debug:" + "licenseid - " + license_id)
        if not self.spdx_license_list: # list is empty
            self.get_spdx_license_list()
        if not self.spdx_license_exception_list: # list is empty
            self.get_spdx_license_exception_list()

        for spdx_license in self.spdx_license_list['licenses']:
            if spdx_license['licenseId'] == license_id:
                return spdx_license['detailsUrl']
        for spdx_license_exception in self.spdx_license_exception_list['exceptions']:
            if spdx_license_exception['licenseExceptionId'] == license_id:
                return SPDX_LICENSE_URL_PREFIX + str(spdx_license_exception['reference']).replace("./", "")

        return None
    
    def get_spdx_license_details(self, details_url):
        r = requests.get(details_url)
        detalis_license = r.json()

        # convert to license form if detalis_license is license exception
        if "licenseId" not in detalis_license:
            detalis_license['licenseId'] = detalis_license['licenseExceptionId']
        if 'licenseText' not in detalis_license:
            detalis_license['licenseText'] = detalis_license['licenseExceptionText']
        if 'licenseTextHtml' not in detalis_license:
            detalis_license['licenseTextHtml'] = detalis_license['exceptionTextHtml']

        return detalis_license


#!/usr/bin/env python3

# SPDX-FileCopyrightText: Copyright 2022 SK TELECOM CO., LTD. <haksung@sk.com>
# SPDX-License-Identifier: Apache-2.0
from onot.parsing import excel
from onot.parsing import rdf_xml

def parse_file(infile):
    print("debug: " + "parse_file - " + infile)
    if infile.endswith(".xls") or infile.endswith(".xlsx"):
        parsing_module = excel
    elif infile.endswith(".rdf") or infile.endswith(".rdf.xml"):
        parsing_module = rdf_xml
    else:
        raise Exception("FileType Not Supported" + str(infile))
    
    p = parsing_module.Parser(infile)
    return p.parse(infile)
    # with open(infile) as f:
    #     return p.parse(f)
    


#!/usr/bin/env python3

# SPDX-FileCopyrightText: Copyright 2022 SK TELECOM CO., LTD. <haksung@sk.com>
# SPDX-License-Identifier: Apache-2.0
from onot.generating import html

def generate_notice(doc, ext):
    if ext == 'html':
        file_type = html
    g = file_type.Generator()
    g.generate(doc)
    


#!/usr/bin/env python3

# SPDX-FileCopyrightText: Copyright 2022 SK TELECOM CO., LTD. <haksung@sk.com>
# SPDX-License-Identifier: Apache-2.0

from onot.tools import create_notice


def main():
    create_notice.main()


if __name__ == '__main__':
    main()


#!/usr/bin/env python3

# SPDX-FileCopyrightText: Copyright 2022 SK TELECOM CO., LTD. <haksung@sk.com>
# SPDX-FileCopyrightText: Copyright (c) 2022 Kakao Corp. https://www.kakaocorp.com
#
# SPDX-License-Identifier: Apache-2.0

import os.path
import re
from abc import abstractmethod, ABC
from onot.parsing import spdx_license

class AbstractParser(ABC):

    def __init__(self):
        self.doc = {
            "name": "",
            "creation_info": {},
            "packages": [],
            "extracted_license": []
        }

    @abstractmethod
    def validate_file(self):
        pass

    @abstractmethod
    def document_info(self):
        # set self.doc["name"], self.doc["creation_info"]
        pass

    @abstractmethod
    def package_info(self):
        # set self.doc["packages"]
        pass

    @abstractmethod
    def per_file_info(self):
        # set self.doc["packages"]
        pass

    @abstractmethod
    def extracted_license_info(self):
        # set self.doc["extracted_license"]
        pass

    def append_package(self, package_to_be_appended):
        # Check if there are duplicate packages already appended
        if any(package["name"] == package_to_be_appended["name"] and package["versionInfo"] == package_to_be_appended["versionInfo"] for package in self.doc["packages"]):
            return
        self.doc["packages"].append(package_to_be_appended)

    def extract_package_name_and_package_version(self, filename):
        def remove_meaningless_word(_filename):
            _filename = _filename.replace("-sources", "")
            _filename = _filename.replace("-RELEASE", "")
            _filename = _filename.replace("-SNAPSHOT", "")
            return _filename

        filename = os.path.splitext(os.path.basename(filename))[0]
        filename = remove_meaningless_word(filename)
        version = re.search(r"-\d", filename)

        if version:
            return filename[:version.start()], filename[version.start() + 1:]
        else:
            return filename, ""

    def remove_enclosed_parentheses(self, expression):
        # if the overall expression is enclosed in parentheses, remove the parentheses.
        if str(expression).startswith("(") and str(expression).endswith(")"):
            return expression[1:len(expression)-1]
        return expression

    def parse_license_expression(self, license_expression):
        license_names = re.split(r'OR|AND|WITH', str(license_expression))
        # remove (, ), blank
        return list(map(lambda license: license.replace("(", "").replace(")", "").strip(), license_names))

    def add_package_info_if_license_exist(self, license_name, package_name, package_version):
        # then, update data just to add package info
        for index, license in enumerate(self.licenses):
            if license["licenseId"] == license_name:
                # then, update data just to add package info
                target_license = license
                target_package = license['packages']
                target_package.append((package_name, package_version))
                target_license['packages'] = target_package
                self.licenses[index] = target_license
                return True
        return False

    def get_details_license(self, license_name):
        sl = spdx_license.SPDX_License()

        # check whether the licenseId is existed in the spdx license list
        details_url = sl.get_spdx_license_detailsUrl(license_name)
        print("debug: " + str(details_url))
        if details_url is not None:
            # if so, get the license text from spdx repo : "https://spdx.org/licenses/[LICENSE_ID].json"
            details_license = sl.get_spdx_license_details(details_url)
        else:
            # check the license is in the Extracted License Info sheet
            existed_in_extracted_license = False
            for index, extracted_license in enumerate(self.doc['extracted_license']):
                if extracted_license['identifier'] == license_name:
                    details_license = {
                        'licenseId': extracted_license['identifier'],
                        'name': extracted_license['licenseName'],
                        'licenseText': extracted_license['extractedText'],
                        'licenseTextHtml': None
                    }
                    existed_in_extracted_license = True
                    break
            if existed_in_extracted_license is False:
                raise ValueError("This license is not in the spdx license list. then it should be in the Extracted License Info sheet: " + license_name)

                # Check if there are duplicate licenses already appended
        return details_license

    def append_details_license(self, detalis_license, package_name, package_version):
        packages = []
        packages.append((package_name, package_version))
        license = {
            "name": detalis_license['name'],
            "licenseId": detalis_license['licenseId'],
            "packages": packages,
            "licenseText": detalis_license['licenseText'],
            "licenseTextHtml": detalis_license['licenseTextHtml']
        }
        self.licenses.append(license)

    def license_info(self):
        self.licenses = []

        # get necessary license info
        for package in self.doc["packages"]:
            package_name = package["name"]
            package_version = str(package["versionInfo"])
            license_expression = package["licenseConcluded"]
            if license_expression is None:
                license_expression = package["licenseDeclared"]

            license_names = self.parse_license_expression(license_expression)
            for license_name in license_names:
                if self.add_package_info_if_license_exist(license_name, package_name, package_version) is False:
                    details_license = self.get_details_license(license_name)
                    self.append_details_license(details_license, package_name, package_version)

        self.doc["licenses"] = self.licenses


    def load_doc(self, file):

        # Document info
        self.document_info()

        # Package info
        self.package_info()

        # Per File Info
        self.per_file_info()

        # Extracted License Info
        self.extracted_license_info()

        # License info
        self.license_info()

        return self.doc

    def parse(self, file):
        self.validate_file(file)
        doc = self.load_doc(file)
        return doc


#!/usr/bin/env python3

# SPDX-FileCopyrightText: Copyright 2022 SK TELECOM CO., LTD. <haksung@sk.com>
# SPDX-License-Identifier: Apache-2.0

import click
from onot.parsing.parse import parse_file
from onot.generating.generate import generate_notice

# override the help option so that you can also see help with -h
CONTEXT_SETTINGS = dict(help_option_names=['-h', '--help'])

@click.command(context_settings=CONTEXT_SETTINGS, help="This creates an OSS Notice file based don the excel format SPDX document.\n ex) onot -i sample/SPDXRdfExample-v2.1.xlsx -o html")
# @click.argument('input')
@click.option('-i', '--input', type=click.STRING, required=True, help="Write the input file name.")
@click.option('-o', '--output_format', type=click.STRING, required=True, help="Write the output file format.")
def main(input, output_format):
    """
    This creates the packages of the spdx document as oss notice.

    Params
    ----------
    input: str
        excel format spdx document file name
    html_format: bool
        if True, only create html format oss notice
    text_format: bool
        if True, only create text format oss notice
    """
    print("debug: " + 'called create')
    print("debug: " + "input - " + input)
    print("debug: " + "output - " + output_format)

    if output_format != 'html': 
        print("Sorry! Current version only supports html type output.")
    else:
        # parse excel file
        doc = parse_file(input)

        # generate html format oss notice
        generate_notice(doc, 'html')

if __name__ == "__main__":
    main()


#!/usr/bin/env python3

# SPDX-FileCopyrightText: Copyright 2022 SK TELECOM CO., LTD. <haksung@sk.com>
# SPDX-License-Identifier: Apache-2.0
import unittest
from onot.parsing import excel

SAMPLE_FILE = 'sample/SPDXRdfExample-v2.1.xlsx'

class TestClass(unittest.TestCase):
    def setUp(self):
        self.instance = excel.Parser(SAMPLE_FILE)
        
    def tearDown(self):
        # self.instance.dispose()
        del self.instance
        
    def test_result(self):
        result=self.instance.parse(SAMPLE_FILE)
        # self.assertEqual(result, True)
        
    # def test_raise_exception(self):
    #     self.assertRaises(Exception, lambda: self.instance.example_method2())


# !/usr/bin/env python3

# SPDX-FileCopyrightText: Copyright 2022 SK TELECOM CO., LTD. <haksung@sk.com>
# SPDX-FileCopyrightText: Copyright (c) 2022 Kakao Corp. https://www.kakaocorp.com
#
# SPDX-License-Identifier: Apache-2.0
import unittest
from onot.parsing import spdx_license

class TestGetSPDXLicenseCase(unittest.TestCase):
    # license and license exception from spdx-spec/v2.3
    # https://spdx.github.io/spdx-spec/v2.3/SPDX-license-list/#a1-licenses-with-short-identifiers
    # https://spdx.github.io/spdx-spec/v2.3/SPDX-license-list/#a2-exceptions-list
    licenses = ['Apache-2.0', 'BSD-3-Clause', 'EPL-2.0', 'LGPL-2.1-only', 'MPL-2.0',
                'Autoconf-exception-2.0', 'Bison-exception-2.2', 'GCC-exception-2.0']

    def setUp(self):
        self.instance = spdx_license.SPDX_License()

    def tearDown(self):
        # self.instance.dispose()
        del self.instance

    def test_result(self):
        for license in self.licenses:
            with self.subTest(license=license):
                details_url = self.instance.get_spdx_license_detailsUrl(license)
                license_details = self.instance.get_spdx_license_details(details_url)
                self.assertTrue(license_details['licenseId'] is not None or license_details['licenseExceptionId'] is not None)




#!/usr/bin/env python3

# SPDX-FileCopyrightText: Copyright 2022 SK TELECOM CO., LTD. <haksung@sk.com>
# SPDX-License-Identifier: Apache-2.0

import setuptools

VERSION = '0.0.3'

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="onot",
    version=VERSION,
    author="Haksung Jang",
    author_email="hakssung@gmail.com",
    description="Generate open source software notice based on the SPDX document",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/sktelecom/onot",
    packages=setuptools.find_packages(),
    install_requires=[
        'click',
        'openpyxl',
        'pandas',
        'requests',
    ],
    entry_points={
        "console_scripts": [
            "onot=onot.__main__:main"
        ]
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
)


#!/usr/bin/env python3

# SPDX-FileCopyrightText: Copyright 2022 SK TELECOM CO., LTD. <haksung@sk.com>
# SPDX-FileCopyrightText: Copyright (c) 2022 Kakao Corp. https://www.kakaocorp.com
#
# SPDX-License-Identifier: Apache-2.0

import unittest

from onot.parsing import rdf_xml

SAMPLE_FILE = 'sample/SPDXRdfExample-v2.3.rdf.xml'

class TestParsingRdfXmlClass(unittest.TestCase):
    def setUp(self):
        self.instance = rdf_xml.Parser(SAMPLE_FILE)

    def tearDown(self):
        # self.instance.dispose()
        del self.instance

    def test_result(self):
        result = self.instance.parse(SAMPLE_FILE)
